@@ -10,7 +10,7 @@ package Affix {    # 'FFI' is my middle name!
     use Text::ParseWords;
     use Carp qw[];
     use vars qw[@EXPORT_OK @EXPORT %EXPORT_TAGS];
-    our $VERSION = '0.10_01';
+    our $VERSION = '0.10_02';
     use XSLoader;
     XSLoader::load( __PACKAGE__, $VERSION );
     #
@@ -55,7 +55,7 @@ package Affix {    # 'FFI' is my middle name!
             #ddx [ $lib, $_delay{$sub}[3], $sig, $ret, $_delay{$sub}[6] ];
             my $cv = affix( $lib, $_delay{$sub}[3], $sig, $ret, $_delay{$sub}[6] );
             Carp::croak 'Undefined subroutine &' . $_delay{$sub}[6] unless $cv;
-            delete $_delay{$sub};
+            delete $_delay{$sub} if defined $_delay{$sub};
             return &$cv;
         }
 
