@@ -45,7 +45,7 @@ massive retval = {.B = true,
                   .d = 1.246,
                   .p = &_p,
                   .Z = "Just a little test",
-                  .u = 5,
+                  .u = {.i = 5},
                   .A = {.i = 50}};
 
 DLLEXPORT massive *massive_ptr() {
@@ -77,8 +77,8 @@ DLLEXPORT bool sptr(massive *sptr) {
 }
 
 DLLEXPORT char *dbl_ptr(double *dbl) {
-    warn("# dbl_ptr( ... )");
-    warn("# dbl == %f", *dbl);
+    // warn("# dbl_ptr( ... )");
+    // warn("# dbl == %f", *dbl);
     if (dbl == NULL)
         return "NULL";
     else if (*dbl == 0) {
